"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.comparatorBuilder = void 0;
const comparatorBuilder = () => {
    const comparators = [];
    const builder = {
        build() {
            return (a, b) => {
                for (const comparator of comparators) {
                    const result = comparator(a, b);
                    if (result !== 0) {
                        return result;
                    }
                }
                return 0;
            };
        },
        num(accessor) {
            comparators.push((a, b) => {
                const aValue = accessor(a);
                const bValue = accessor(b);
                if (bValue === undefined && aValue === undefined) {
                    return 0;
                }
                if (bValue === undefined) {
                    return -1;
                }
                if (aValue === undefined) {
                    return 1;
                }
                return aValue - bValue;
            });
            return builder;
        },
        str(accessor) {
            comparators.push((a, b) => {
                const aValue = accessor(a);
                const bValue = accessor(b);
                if (bValue === undefined && aValue === undefined) {
                    return 0;
                }
                if (bValue === undefined) {
                    return -1;
                }
                if (aValue === undefined) {
                    return 1;
                }
                return aValue.localeCompare(bValue);
            });
            return builder;
        },
    };
    return builder;
};
exports.comparatorBuilder = comparatorBuilder;
//# sourceMappingURL=comparator.js.map