"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.doubleLinkedList = exports.singleLinkedList = void 0;
class ALinkedList {
    constructor() {
        this.count = 0;
        this._head = undefined;
    }
    get head() {
        var _a;
        return (_a = this._head) === null || _a === void 0 ? void 0 : _a.value;
    }
    insertAt(index, value) {
        const [prev, next] = this.around(index);
        const node = this.buildNode(prev, value, next);
        this.relink(prev, node, next);
        this.count++;
    }
    around(index) {
        var _a;
        if (index < -1) {
            return [undefined, undefined, undefined];
        }
        if (index === -1) {
            return [undefined, undefined, this._head];
        }
        if (index === 0) {
            return [undefined, this._head, (_a = this._head) === null || _a === void 0 ? void 0 : _a.next];
        }
        if (index === this.count - 1) {
            return [...this.last2Nodes(), undefined];
        }
        if (index === this.count) {
            return [this.last2Nodes()[1], undefined, undefined];
        }
        if (index > this.count) {
            return [undefined, undefined, undefined];
        }
        let i = 0;
        let current = this._head;
        let prev = undefined;
        while (i < index && current != null) {
            i++;
            prev = current;
            current = current.next;
        }
        return [prev, current, current === null || current === void 0 ? void 0 : current.next];
    }
    get isEmpty() {
        return this.count === 0;
    }
    get length() {
        return this.count;
    }
    at(index) {
        var _a;
        return (_a = this.around(index)[1]) === null || _a === void 0 ? void 0 : _a.value;
    }
    pop() {
        if (this.count === 0) {
            return undefined;
        }
        const [penultimate, last] = this.last2Nodes();
        this.relink(penultimate, undefined, undefined);
        this.count--;
        return last === null || last === void 0 ? void 0 : last.value;
    }
    push(value) {
        const [, last] = this.last2Nodes();
        const node = this.buildNode(last, value, undefined);
        this.relink(last, node, undefined);
        this.count++;
    }
    shift() {
        if (this._head == null) {
            return undefined;
        }
        const value = this._head.value;
        this.relink(undefined, undefined, this._head.next);
        this.count--;
        return value;
    }
    toArray() {
        return Array.from(this.values());
    }
    unshift(value) {
        const node = this.buildNode(undefined, value, this._head);
        this.relink(undefined, node, this._head);
        this.count++;
    }
    *values() {
        let current = this._head;
        while (current != null) {
            yield current.value;
            current = current.next;
        }
    }
    *valuesAndIndexes() {
        let current = this._head;
        let index = 0;
        while (current != null) {
            yield [current.value, index];
            current = current.next;
            index++;
        }
    }
    *windows() {
        let current = this._head;
        let index = 0;
        let prev = undefined;
        while (current != null) {
            const next = current.next;
            yield [prev, current, next, index];
            prev = current;
            current = next;
            index++;
        }
    }
    walkValues(decide, windows = this.windows()) {
        let adjustedPrev = undefined;
        for (const [prev, current, next, index] of windows) {
            const decision = decide(current.value, index);
            if (decision.deleteItem) {
                this.relink(adjustedPrev !== null && adjustedPrev !== void 0 ? adjustedPrev : prev, undefined, next);
                this.count--;
            }
            if (decision.insertBefore != null) {
                const inserted = this.buildNode(adjustedPrev !== null && adjustedPrev !== void 0 ? adjustedPrev : prev, decision.insertBefore, current);
                this.relink(adjustedPrev !== null && adjustedPrev !== void 0 ? adjustedPrev : prev, inserted, current);
                this.count++;
            }
            if (decision.insertAfter != null) {
                const inserted = this.buildNode(current, decision.insertAfter, next);
                this.relink(current, inserted, next);
                this.count++;
                adjustedPrev = inserted;
            }
            else {
                adjustedPrev = undefined;
            }
            if (!decision.keepWalking) {
                break;
            }
        }
    }
    walk(decide) {
        this.walkValues(decide);
    }
}
class SingleLinkedListImpl extends ALinkedList {
    get [Symbol.toStringTag]() {
        return `SingleLinkedList+${this.count}`;
    }
    buildNode(_prev, value, next) {
        return { next, value };
    }
    last2Nodes() {
        let last = this._head;
        if (last == null) {
            return [undefined, undefined];
        }
        let penultimate = undefined;
        while (last.next != null) {
            penultimate = last;
            last = last.next;
        }
        return [penultimate, last];
    }
    relink(prev, node, next) {
        if (prev == null) {
            this._head = node !== null && node !== void 0 ? node : next;
        }
        else {
            prev.next = node !== null && node !== void 0 ? node : next;
        }
    }
}
const singleLinkedList = () => {
    return new SingleLinkedListImpl();
};
exports.singleLinkedList = singleLinkedList;
class DoubleLinkedListImpl extends SingleLinkedListImpl {
    constructor() {
        super(...arguments);
        this._tail = undefined;
    }
    get [Symbol.toStringTag]() {
        return `DoubleLinkedList+${this.count}`;
    }
    get tail() {
        var _a;
        return (_a = this._tail) === null || _a === void 0 ? void 0 : _a.value;
    }
    reverseIndex(index) {
        return this.count - index - 1;
    }
    around(index) {
        const reverseIndex = this.reverseIndex(index);
        if (reverseIndex >= index) {
            return super.around(index);
        }
        let i = 0;
        let current = this._tail;
        while (i < reverseIndex && current != null) {
            i++;
            current = current.prev;
        }
        return [current === null || current === void 0 ? void 0 : current.prev, current, current === null || current === void 0 ? void 0 : current.next];
    }
    buildNode(prev, value, next) {
        return { next, prev, value };
    }
    last2Nodes() {
        var _a;
        return [(_a = this._tail) === null || _a === void 0 ? void 0 : _a.prev, this._tail];
    }
    relink(prev, node, next) {
        super.relink(prev, node, next);
        if (next == null) {
            this._tail = node !== null && node !== void 0 ? node : prev;
        }
        else {
            next.prev = node !== null && node !== void 0 ? node : prev;
        }
    }
    *reverseValues() {
        let current = this._tail;
        while (current != null) {
            yield current.value;
            current = current.prev;
        }
    }
    *reverseValuesAndIndexes() {
        let current = this._tail;
        let index = this.count - 1;
        while (current != null) {
            yield [current.value, index];
            current = current.prev;
            index--;
        }
    }
    reverseWalk(block) {
        this.walkValues(block, this.reverseWindows());
    }
    *reverseWindows() {
        let current = this._tail;
        let index = this.count - 1;
        while (current != null) {
            const prev = current.prev;
            yield [prev, current, current.next, index];
            current = prev;
            index--;
        }
    }
}
const doubleLinkedList = () => {
    return new DoubleLinkedListImpl();
};
exports.doubleLinkedList = doubleLinkedList;
//# sourceMappingURL=linked-list.js.map