export const binaryIndexOf = (value, items, comparator, { initialLeft = 0, initialRight = items.length, firstNonUnique, rangeCheck } = {}) => {
    let left = initialLeft;
    let right = initialRight;
    let comparisons = 0;
    if (left >= right) {
        return { before: right, comparisons: 0, exists: false };
    }
    const compareAt = (index) => {
        const other = items[index];
        const compared = other === value ? 0 : comparator(value, other);
        comparisons++;
        return compared;
    };
    if (rangeCheck) {
        const beforeLeft = compareAt(left);
        if (beforeLeft < 0) {
            return { before: left, comparisons, exists: false };
        }
        if (beforeLeft === 0) {
            return { comparisons, exists: true, index: left };
        }
        left++;
        if (left < right) {
            const afterRight = compareAt(right - 1);
            if (afterRight > 0) {
                return { before: right, comparisons, exists: false };
            }
            if (afterRight === 0) {
                if (firstNonUnique) {
                    while (right > 1 && compareAt(right - 2) === 0) {
                        right--;
                    }
                }
                return { comparisons, exists: true, index: right - 1 };
            }
            right--;
        }
    }
    while (left < right) {
        let index = (left + right) >> 1;
        const compared = compareAt(index);
        if (compared === 0) {
            if (firstNonUnique) {
                while (index > 0 && compareAt(index - 1) === 0) {
                    index--;
                }
            }
            return { comparisons, exists: true, index };
        }
        else if (compared < 0) {
            right = index;
        }
        else {
            left = index + 1;
        }
    }
    return { before: right, comparisons, exists: false };
};
//# sourceMappingURL=binary-index-of.js.map