import { singleLinkedList } from "./linked-list.js";
export class PriorityLinkedList {
    comparator;
    list = singleLinkedList();
    constructor(comparator) {
        this.comparator = comparator;
    }
    add(item) {
        if (this.list.isEmpty) {
            this.list.unshift(item);
        }
        else {
            const lastIndex = this.list.length - 1;
            this.list.walk((other, index) => {
                const comparison = this.comparator(item, other);
                if (other === item) {
                    return { keepWalking: false };
                }
                if (comparison < 0) {
                    return { insertBefore: item, keepWalking: false };
                }
                if (index === lastIndex) {
                    return { insertAfter: item, keepWalking: false };
                }
                return { keepWalking: true };
            });
        }
    }
    get length() {
        return this.list.length;
    }
    peek() {
        return this.list.head;
    }
    remove(item) {
        this.list.walk((other) => {
            const comparison = this.comparator(item, other);
            if (comparison === 0) {
                return { deleteItem: true, keepWalking: true };
            }
            if (comparison < 0) {
                return { keepWalking: false };
            }
            return { keepWalking: true };
        });
    }
    take() {
        return this.list.shift();
    }
    toArray() {
        return this.list.toArray();
    }
    values() {
        return this.list.values();
    }
}
//# sourceMappingURL=priority-linked-list.js.map