import type { Comparator } from "@rickosborne/typical";
export type SearchResult = {
    before?: undefined;
    comparisons: number;
    exists: true;
    index: number;
} | {
    before: number;
    comparisons: number;
    exists: false;
    index?: undefined;
};
export type BinaryIndexOfConfig = {
    /**
     * The values may not be unique, and the result should be the
     * index of the first occurrence.
     * When a value is found, step back and compare until the first
     * occurrence of that value is found.  Will always cause extra
     * comparisons.
     * @default {false}
     */
    firstNonUnique?: boolean;
    /**
     * Left-most index to check.  Inclusive.
     * @default {0}
     */
    initialLeft?: number;
    /**
     * Right-most index, considered out-of-bounds.  Exclusive.
     * @default The length of the array.
     */
    initialRight?: number;
    /**
     * Perform an initial check against the first and last elements.
     * Can speed up searches where you expect the value to be less
     * than or greater than all the elements, such as when you mostly
     * insert at the front or append to the back.
     * But it can just add extra comparisons for small arrays, or
     * arrays where most values will be in the middle.
     * @default {false}
     */
    rangeCheck?: boolean;
};
export declare const binaryIndexOf: <T>(value: T, items: T[], comparator: Comparator<T>, { initialLeft, initialRight, firstNonUnique, rangeCheck }?: BinaryIndexOfConfig) => SearchResult;
