"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PriorityArray = void 0;
const binary_index_of_js_1 = require("./binary-index-of.js");
class PriorityArray {
    comparator;
    items = [];
    constructor(comparator) {
        this.comparator = comparator;
    }
    get [Symbol.toStringTag]() {
        return `PriorityArray+${this.items.length}`;
    }
    get length() {
        return this.items.length;
    }
    add(value) {
        const { exists, before } = (0, binary_index_of_js_1.binaryIndexOf)(value, this.items, this.comparator);
        if (exists) {
            return;
        }
        if (before === 0) {
            this.items.unshift(value);
        }
        else if (before === this.items.length) {
            this.items.push(value);
        }
        else {
            this.items.splice(before, 0, value);
        }
    }
    peek() {
        return this.items[0];
    }
    remove(item) {
        for (let i = this.items.length - 1; i >= 0; i--) {
            const other = this.items[i];
            const comparison = this.comparator(item, other);
            if (comparison === 0) {
                this.items.splice(i, 1);
            }
            else if (comparison > 0) {
                break;
            }
        }
    }
    take() {
        return this.items.shift();
    }
    toArray() {
        return this.items.slice();
    }
    *values() {
        for (const item of this.items) {
            yield item;
        }
    }
}
exports.PriorityArray = PriorityArray;
//# sourceMappingURL=priority-array.js.map